@@ -1,4 +1,4 @@
-# Copyright 2001-2014, Paul Johnson (paul@pjcj.net)
+# Copyright 2001-2018, Paul Johnson (paul@pjcj.net)
 
 # This software is free.  It is licensed under the same terms as Perl itself.
 
@@ -10,18 +10,20 @@ package Devel::Cover;
 use strict;
 use warnings;
 
-our $VERSION = '1.17'; # VERSION
-our $LVERSION = do { no warnings; eval '$VERSION' || "0.001" };  # for dev
+our $VERSION;
+BEGIN {
+our $VERSION = '1.31'; # VERSION
+}
 
 use DynaLoader ();
 our @ISA = "DynaLoader";
 
-# sub Pod::Coverage::TRACE_ALL () { 1 }
-
 use Devel::Cover::DB;
 use Devel::Cover::DB::Digests;
 use Devel::Cover::Inc;
 
+BEGIN { $VERSION //= $Devel::Cover::Inc::VERSION }
+
 use B qw( class ppname main_cv main_start main_root walksymtable OPf_KIDS );
 use B::Debug;
 use B::Deparse;
@@ -35,9 +37,9 @@ use Devel::Cover::Dumper;
 use Devel::Cover::Util "remove_contained_paths";
 
 BEGIN {
-    # Use Pod::Coverage if it is available.
+    # Use Pod::Coverage if it is available
     eval "use Pod::Coverage 0.06";
-    # If there is any error other than a failure to locate, report it.
+    # If there is any error other than a failure to locate, report it
     die $@ if $@ && $@ !~ m/Can't locate Pod\/Coverage.+pm in \@INC/;
 
     # We'll prefer Pod::Coverage::CountParents
@@ -46,45 +48,47 @@ BEGIN {
 }
 
 # $SIG{__DIE__} = \&Carp::confess;
+# sub Pod::Coverage::TRACE_ALL () { 1 }
 
-my $Initialised;                         # import() has been called.
+my $Initialised;                         # import() has been called
 
 my $Dir;                                 # Directory in which coverage will be
-                                         # collected.
-my $DB             = "cover_db";         # DB name.
-my $Merge          = 1;                  # Merge databases.
-my $Summary        = 1;                  # Output coverage summary.
-my $Subs_only      = 0;                  # Coverage only for sub bodies.
-my $Self_cover_run = 0;                  # Covering Devel::Cover now.
+                                         # collected
+my $DB             = "cover_db";         # DB name
+my $Merge          = 1;                  # Merge databases
+my $Summary        = 1;                  # Output coverage summary
+my $Subs_only      = 0;                  # Coverage only for sub bodies
+my $Self_cover_run = 0;                  # Covering Devel::Cover now
+my $Loose_perms    = 0;                  # Use loose permissions in the cover DB
 
-my @Ignore;                              # Packages to ignore.
-my @Inc;                                 # Original @INC to ignore.
-my @Select;                              # Packages to select.
-my @Ignore_re;                           # Packages to ignore.
-my @Inc_re;                              # Original @INC to ignore.
-my @Select_re;                           # Packages to select.
+my @Ignore;                              # Packages to ignore
+my @Inc;                                 # Original @INC to ignore
+my @Select;                              # Packages to select
+my @Ignore_re;                           # Packages to ignore
+my @Inc_re;                              # Original @INC to ignore
+my @Select_re;                           # Packages to select
 
 my $Pod = $INC{"Pod/Coverage/CountParents.pm"} ? "Pod::Coverage::CountParents"
         : $INC{"Pod/Coverage.pm"}              ? "Pod::Coverage"
-        : "";                            # Type of pod coverage available.
-my %Pod;                                 # Pod coverage data.
+        : "";                            # Type of pod coverage available
+my %Pod;                                 # Pod coverage data
 
-my @Cvs;                                 # All the Cvs we want to cover.
-my %Cvs;                                 # All the Cvs we want to cover.
-my @Subs;                                # All the subs we want to cover.
-my $Cv;                                  # Cv we are looking in.
-my $Sub_name;                            # Name of the sub we are looking in.
-my $Sub_count;                           # Count for multiple subs on same line.
+my @Cvs;                                 # All the Cvs we want to cover
+my %Cvs;                                 # All the Cvs we want to cover
+my @Subs;                                # All the subs we want to cover
+my $Cv;                                  # Cv we are looking in
+my $Sub_name;                            # Name of the sub we are looking in
+my $Sub_count;                           # Count for multiple subs on same line
 
-my $Coverage;                            # Raw coverage data.
-my $Structure;                           # Structure of the files.
-my $Digests;                             # Digests of the files.
+my $Coverage;                            # Raw coverage data
+my $Structure;                           # Structure of the files
+my $Digests;                             # Digests of the files
 
-my %Criteria;                            # Names of coverage criteria.
-my %Coverage;                            # Coverage criteria to collect.
-my %Coverage_options;                    # Options for overage criteria.
+my %Criteria;                            # Names of coverage criteria
+my %Coverage;                            # Coverage criteria to collect
+my %Coverage_options;                    # Options for overage criteria
 
-my %Run;                                 # Data collected from the run.
+my %Run;                                 # Data collected from the run
 
 my $Const_right = qr/^(?:const|s?refgen|gelem|die|undef|bless|anon(?:list|hash)|
                        scalar|return|last|next|redo|goto)$/x;
@@ -95,11 +99,11 @@ use vars '$File',                        # Last filename we saw.  (localised)
          '$Collect',                     # Whether or not we are collecting
                                          # coverage data.  We make two passes
                                          # over conditions.  (localised)
-         '%Files',                       # Whether we are interested in files.
-                                         # Used in runops function.
-         '$Replace_ops',                 # Whether we are replacing ops.
-         '$Silent',                      # Output nothing. Can be used anywhere.
-         '$Self_cover';                  # Coverage of Devel::Cover.
+         '%Files',                       # Whether we are interested in files
+                                         # Used in runops function
+         '$Replace_ops',                 # Whether we are replacing ops
+         '$Silent',                      # Output nothing. Can be used anywhere
+         '$Self_cover';                  # Coverage of Devel::Cover
 
 BEGIN {
     ($File, $Line, $Collect) = ("", 0, 1);
@@ -107,11 +111,52 @@ BEGIN {
               ($ENV{PERL5OPT}              || "") =~ /Devel::Cover/;
     *OUT = $ENV{DEVEL_COVER_DEBUG} ? *STDERR : *STDOUT;
 
+    if ($] < 5.010000 && !$ENV{DEVEL_COVER_UNSUPPORTED}) {
+        my $v = $] < 5.008001 ? "1.22" : "1.23";
+        print <<EOM;
+
+================================================================================
+
+                                   IMPORTANT
+                                   ---------
+
+Devel::Cover $VERSION is not supported on perl $].  The last version of
+Devel::Cover which was supported was version $v.  This version may not work.
+I have not tested it.  If it does work it will not be fully functional.
+
+If you decide to use it anyway, you are on your own.  If it works at all, there
+will be some constructs for which coverage will not be collected, and you may
+well encounter bugs which have been fixed in subsequent versions of perl.
+EOM
+
+        print <<EOM if $^O eq "MSWin32";
+
+And things are even worse under Windows.  You may well find random bugs of
+various severities.
+EOM
+        print <<EOM;
+
+If you are actually using this version of Devel::Cover with perl $], please let
+me know.  I don't want to know if you are just testing Devel::Cover, only if you
+are seriously using this version to do code coverage analysis of real code.  If
+I get no reports of such usage then I will remove support and delete the
+workarounds for versions of perl below 5.10.0.
+
+In order to use this version of Devel::Cover with perl $] you must set the
+environment variable \$DEVEL_COVER_UNSUPPORTED
+
+================================================================================
+
+EOM
+
+        die "Exiting";
+    }
+
     if ($^X =~ /(apache2|httpd)$/) {
         # mod_perl < 2.0.8
         @Inc = @Devel::Cover::Inc::Inc;
     } else {
-        # Can't get @INC via eval `` in taint mode, revert to default value.
+        # Can't get @INC via eval `` in taint mode, revert to default value
         if (${^TAINT}) {
             @Inc = @Devel::Cover::Inc::Inc;
         } else {
@@ -145,7 +190,7 @@ BEGIN {
     $^P |= 0x004 | 0x100;
 }
 
-sub version { $LVERSION }
+sub version { $VERSION }
 
 if (0 && $Config{useithreads}) {
     eval "use threads";
@@ -186,8 +231,9 @@ if (0 && $Config{useithreads}) {
         my $sub       = shift;
         my $wantarray = wantarray;
 
-        $new->($class,
-               sub {
+        $new->(
+            $class,
+            sub {
                    print STDERR "Starting thread\n";
                    set_coverage(keys %Coverage);
                    my $ret = [ $sub->(@_) ];
@@ -195,9 +241,9 @@ if (0 && $Config{useithreads}) {
                    report() if $Initialised;
                    print STDERR "Ended thread\n";
                    $wantarray ? @{$ret} : $ret->[0];
-               },
-               @_
-              );
+            },
+            @_
+        );
     };
 }
 
@@ -224,7 +270,7 @@ EOM
         @coverage = get_coverage();
         my $last = pop @coverage || "";
 
-        print OUT __PACKAGE__, " $LVERSION: Collecting coverage data for ",
+        print OUT __PACKAGE__, " $VERSION: Collecting coverage data for ",
               join(", ", @coverage),
               @coverage ? " and " : "",
               "$last.\n",
@@ -288,7 +334,10 @@ sub import {
 
     my $class = shift;
 
-    my @o = (@_, split ",", $ENV{DEVEL_COVER_OPTIONS} || "");
+    # Die tainting
+    # Anyone using this module can do worse things than messing with tainting
+    my $options = ($ENV{DEVEL_COVER_OPTIONS} || "") =~ /(.*)/ ? $1 : "";
+    my @o = (@_, split ",", $options);
     defined or $_ = "" for @o;
     # print STDERR __PACKAGE__, ": Parsing options from [@o]\n";
 
@@ -302,6 +351,7 @@ sub import {
         /^-silent/      && do { $Silent      = shift @o; next };
         /^-dir/         && do { $Dir         = shift @o; next };
         /^-db/          && do { $DB          = shift @o; next };
+        /^-loose_perms/ && do { $Loose_perms = shift @o; next };
         /^-merge/       && do { $Merge       = shift @o; next };
         /^-summary/     && do { $Summary     = shift @o; next };
         /^-blib/        && do { $blib        = shift @o; next };
@@ -320,7 +370,7 @@ sub import {
 
     if ($blib) {
         eval "use blib";
-        for (@INC) { $_ = $1 if /(.*)/ }  # Die tainting.
+        for (@INC) { $_ = $1 if /(.*)/ }  # Die tainting
         push @Ignore, "^t/", '\\.t$', '^test\\.pl$';
     }
 
@@ -329,19 +379,10 @@ sub import {
     @Ignore_re = map qr/$_/,                           @Ignore;
     @Inc_re    = map $ci ? qr/^\Q$_\//i : qr/^\Q$_\//, @Inc;
 
-    if ($LVERSION > 0.002) {
-        # Usual situation in production from a full release.
-        bootstrap Devel::Cover $LVERSION;
-    } else {
-        # Usual situation in development.
-        bootstrap Devel::Cover;
-    }
+    bootstrap Devel::Cover $VERSION;
 
     if (defined $Dir) {
-        # Die tainting.
-        # Anyone using this module can do worse things than messing with
-        # tainting.
-        $Dir = $1 if $Dir =~ /(.*)/;
+        $Dir = $1 if $Dir =~ /(.*)/;  # Die tainting
         chdir $Dir or die __PACKAGE__ . ": Can't chdir $Dir: $!\n";
     } else {
         $Dir = $1 if Cwd::getcwd() =~ /(.*)/;
@@ -350,6 +391,7 @@ sub import {
     unless (mkdir $DB) {
         die "Can't mkdir $DB: $!" unless -d $DB;
     }
+    chmod 0777, $DB if $Loose_perms;
     $DB = $1 if abs_path($DB) =~ /(.*)/;
     Devel::Cover::DB->delete($DB) unless $Merge;
 
@@ -385,13 +427,14 @@ sub import {
 sub populate_run {
     my $self = shift;
 
-    $Run{OS}   = $^O;
-    $Run{perl} = $] < 5.010 ? join ".", map ord, split //, $^V
-                            : sprintf "%vd", $^V;
-    $Run{dir}  = $Dir;
-    $Run{run}  = $0;
+    $Run{OS}      = $^O;
+    $Run{perl}    = $] < 5.010 ? join ".", map ord, split //, $^V
+                               : sprintf "%vd", $^V;
+    $Run{dir}     = $Dir;
+    $Run{run}     = $0;
+    $Run{name}    = $Dir;
+    $Run{version} = "unknown";
 
-    my $version;
     my $mymeta = "$Dir/MYMETA.json";
     if (-e $mymeta) {
         eval {
@@ -400,9 +443,14 @@ sub populate_run {
             my $json = $io->read($mymeta);
             $Run{$_} = $json->{$_} for qw( name version abstract );
         }
-    } elsif ($Dir =~ m|.*/([^/]+?)(\d+\.\d+)(?:-\w{6})$|) {
-        $Run{name}    = $1;
-        $Run{version} = $2;
+    } elsif ($Dir =~ m|.*/([^/]+)$|) {
+        my $filename = $1;
+        eval {
+            require CPAN::DistnameInfo;
+            my $dinfo     = CPAN::DistnameInfo->new($filename);
+            $Run{name}    = $dinfo->dist;
+            $Run{version} = $dinfo->version;
+        }
     }
 
     $Run{start} = get_elapsed() / 1e6;
@@ -502,6 +550,9 @@ sub normalised_file {
 sub get_location {
     my ($op) = @_;
 
+    # print STDERR "get_location ", $op, "\n";
+    # use Carp "cluck"; cluck("from here");
+    return unless $op->can("file");  # How does this happen?
     $File = $op->file;
     $Line = $op->line;
     # print STDERR "$File:$Line\n";
@@ -529,7 +580,7 @@ my $find_filename = qr/
 /x;
 
 sub use_file {
-    # If we're in global destruction, forget it.
+    # If we're in global destruction, forget it
     return unless $find_filename;
 
     my ($file) = @_;
@@ -710,7 +761,10 @@ sub _report {
     chdir $Dir or die __PACKAGE__ . ": Can't chdir $Dir: $!\n";
 
     $Run{collected} = \@collected;
-    $Structure      = Devel::Cover::DB::Structure->new(base => $DB);
+    $Structure      = Devel::Cover::DB::Structure->new(
+        base        => $DB,
+        loose_perms => $Loose_perms,
+    );
     $Structure->read_all;
     $Structure->add_criteria(@collected);
     # print STDERR "Start structure: ", Dumper $Structure;
@@ -761,15 +815,17 @@ sub _report {
 
     my $run = time . ".$$." . sprintf "%05d", rand 2 ** 16;
     my $cover = Devel::Cover::DB->new(
-        base      => $DB,
-        runs      => { $run => \%Run },
-        structure => $Structure,
+        base        => $DB,
+        runs        => { $run => \%Run },
+        structure   => $Structure,
+        loose_perms => $Loose_perms,
     );
 
     my $dbrun = "$DB/runs";
     unless (mkdir $dbrun) {
         die "Can't mkdir $dbrun $!" unless -d $dbrun;
     }
+    chmod 0777, $dbrun if $Loose_perms;
     $dbrun .= "/$run";
 
     print OUT __PACKAGE__, ": Writing coverage database to $dbrun\n"
@@ -966,9 +1022,14 @@ sub deparse {
         return "" if $name eq "padrange";
 
         unless ($Seen{statement}{$$op} || $Seen{other}{$$op}) {
-            # Collect everything under here.
+            # Collect everything under here
             local ($File, $Line) = ($File, $Line);
             # print STDERR "Collecting $$op under $File:$Line\n";
+            no warnings "redefine";
+            my $use_dumper = $class eq 'SVOP' && $name eq 'const';
+            local *B::Deparse::const = \&B::Deparse::const_dumper
+              if $use_dumper;
+            require Data::Dumper if $use_dumper;
             $deparse = eval { local $^W; $Original{deparse}->($self, @_) };
             $deparse =~ s/^\010+//mg if defined $deparse;
             $deparse = "Deparse error: $@" if $@;
@@ -976,16 +1037,26 @@ sub deparse {
             # print STDERR "Collect Deparse $op $$op => <$deparse>\n";
         }
 
-        # Get the coverage on this op.
+        # Get the coverage on this op
 
         if ($class eq "COP" && $Coverage{statement}) {
             # print STDERR "COP $$op, seen [$Seen{statement}{$$op}]\n";
-            add_statement_cover($op) unless $Seen{statement}{$$op}++;
+            my $nnnext = "";
+            eval {
+                my $next   = $op->next;
+                my $nnext  = $next && $next->next;
+                   $nnnext = $nnext && $nnext->next;
+            };
+            # print STDERR "COP $$op, ", $next, " -> ", $nnext,
+                                              # " -> ", $nnnext, "\n";
+            if ($nnnext) {
+                add_statement_cover($op) unless $Seen{statement}{$$op}++;
+            }
         } elsif (!$null && $name eq "null"
                       && ppname($op->targ) eq "pp_nextstate"
                       && $Coverage{statement}) {
             # If the current op is null, but it was nextstate, we can still
-            # get at the file and line number, but we need to get dirty.
+            # get at the file and line number, but we need to get dirty
 
             bless $op, "B::COP";
             # print STDERR "null $$op, seen [$Seen{statement}{$$op}]\n";
@@ -993,7 +1064,7 @@ sub deparse {
             bless $op, "B::$class";
         } elsif ($Seen{other}{$$op}++) {
             # print STDERR "seen [$Seen{other}{$$op}]\n";
-            return ""  # Only report on each op once.
+            return ""  # Only report on each op once
         } elsif ($name eq "cond_expr") {
             local ($File, $Line) = ($File, $Line);
             my $cond  = $op->first;
@@ -1013,7 +1084,7 @@ sub deparse {
                     my $newtrue = $newcond->sibling;
                     if ($newcond->name eq "lineseq") {
                         # lineseq to ensure correct line numbers in elsif()
-                        # Bug #37302 fixed by change #33710.
+                        # Bug #37302 fixed by change #33710
                         $newcond = $newcond->first->sibling;
                     }
                     # last in chain is OP_AND => no else
@@ -1107,8 +1178,8 @@ sub get_cover {
     ($Sub_name, my $start) = sub_info($cv);
 
     # warn "get_cover: <$Sub_name>\n";
-    return unless defined $Sub_name;  # Only happens within Safe.pm, AFAIK.
-    # return unless length  $Sub_name;  # Only happens with Self_cover, AFAIK.
+    return unless defined $Sub_name;  # Only happens within Safe.pm, AFAIK
+    # return unless length  $Sub_name;  # Only happens with Self_cover, AFAIK
 
     get_location($start) if $start;
     # print STDERR "[[$File:$Line]]\n";
@@ -1127,8 +1198,8 @@ sub get_cover {
         no warnings "uninitialized";
         if ($File eq $Structure->get_file && $Line == $Structure->get_line &&
             $Sub_name eq "__ANON__" && $Structure->get_sub_name eq "__ANON__") {
-            # Merge instances of anonymous subs into one.
-            # TODO - multiple anonymous subs on the same line.
+            # Merge instances of anonymous subs into one
+            # TODO - multiple anonymous subs on the same line
         } else {
             my $count = $Sub_count->{$File}{$Line}{$Sub_name}++;
             $Structure->set_subroutine($Sub_name, $File, $Line, $count);
@@ -1218,7 +1289,7 @@ Devel::Cover - Code coverage metrics for Perl
 
 =head1 VERSION
 
-version 1.17
+version 1.31
 
 =head1 SYNOPSIS
 
@@ -1269,10 +1340,9 @@ If L<Pod::Coverage::CountParents> is available it will be used instead.
 Coverage data for other criteria are not yet collected.
 
 The F<cover> program can be used to generate coverage reports.  Devel::Cover
-ships with a number of different reports including various types of HTML
-output, textual reports, a report to display missing coverage in the same
-format as compilation errors and a report to display coverage information
-within the Vim editor.
+ships with a number of reports including various types of HTML output, textual
+reports, a report to display missing coverage in the same format as compilation
+errors and a report to display coverage information within the Vim editor.
 
 It is possible to add annotations to reports, for example you can add a column
 to an HTML report showing who last changed a line, as determined by git blame.
@@ -1306,15 +1376,22 @@ reported.
 
 =over
 
-=item * Perl 5.6.1 or greater.  Perl 5.8.8 or greater is recommended.
+=item * Perl 5.10.0 or greater.
 
-Perl 5.7 is unsupported.  Perl 5.8.8 or greater is recommended.  Perl 5.8.7
-has problems and may crash.  Whilst Perl 5.6 should mostly work you will
-probably miss out on coverage information which would be available using a
-more modern version and will likely run into bugs in perl.  Devel::Cover
-support for unsupported Perl versions may be removed at any time, but I try to
-keep older versions running provided this does not cause undue difficulty i
-other areas.
+Perl versions 5.6.1, 5.6.2 and 5.8.x may work to an extent but are unsupported.
+Perl 5.8.7 has problems and may crash.
+
+If you want to use an unsupported version you will need to set the environment
+variable $DEVEL_COVER_UNSUPPORTED.  Unsupported versions are also untested.  I
+will consider patches for unsupported versions only if they do not compromise
+the code.  This is a vague, nebulous concept that I will decide on if and when
+necessary.
+
+If you are using an unsupported version, please let me know.  I don't want to
+know if you are just testing Devel::Cover, only if you are seriously using it to
+do code coverage analysis of real code.  If I get no reports of such usage then
+I will remove support and delete the workarounds for versions of perl below
+5.10.0.  I may do that anyway.
 
 Different versions of perl may give slightly different results due to changes
 in the op tree.
@@ -1350,10 +1427,6 @@ installed, it is preferred.
 
 Required if you want to run Devel::Cover's own tests.
 
-=item * L<Test::Warn>
-
-Some of Devel::Cover's own tests require it.
-
 =item * L<Test::Differences>
 
 Needed if the tests fail and you would like nice output telling you why.
@@ -1362,9 +1435,10 @@ Needed if the tests fail and you would like nice output telling you why.
 
 Needed if you want to run cpancover.
 
-=item * L<JSON>, L<JSON::PP> or L<JSON::XS>
+=item * L<JSON::MaybeXS>
 
-JSON is used to store the coverage database if it is available.
+JSON is used to store the coverage database if it is available. JSON::MaybeXS
+will select the best JSON backend installed.
 
 =back
 
@@ -1376,27 +1450,36 @@ also add any options you need at this point.  I would suggest adding this as
 early as possible in your startup script in order to collect as much coverage
 information as possible.
 
+Alternatively, add -MDevel::Cover to the parameters for mod_perl.
+In this example, Devel::Cover will be operating in silent mode.
+
+ PerlSwitches -MDevel::Cover=-silent,1
+
 =head1 OPTIONS
 
  -blib               - "use blib" and ignore files matching \bt/ (default true
-                       iff blib directory exists).
+                       if blib directory exists, false otherwise)
  -coverage criterion - Turn on coverage for the specified criterion.  Criteria
                        include statement, branch, condition, path, subroutine,
-                       pod, time, all and none (default all available).
- -db cover_db        - Store results in coverage db (default ./cover_db).
+                       pod, time, all and none (default all available)
+ -db cover_db        - Store results in coverage db (default ./cover_db)
  -dir path           - Directory in which coverage will be collected (default
-                       cwd).
- -ignore RE          - Set REs of files to ignore (default "/Devel/Cover\b").
- +ignore RE          - Append to REs of files to ignore.
- -inc path           - Set prefixes of files to include (default @INC).
- +inc path           - Append to prefixes of files to include.
- -merge val          - Merge databases, for multiple test benches (default on).
- -select RE          - Set REs of files to select (default none).
- +select RE          - Append to REs of files to select.
- -silent val         - Don't print informational messages (default off).
- -subs_only val      - Only cover code in subroutine bodies (default off).
- -replace_ops val    - Use op replacing rather than runops (default on).
- -summary val        - Print summary information iff val is true (default on).
+                       cwd)
+ -ignore RE          - Set regular expressions for files to ignore (default
+                       "/Devel/Cover\b")
+ +ignore RE          - Append to regular expressions of files to ignore
+ -inc path           - Set prefixes of files to include (default @INC)
+ +inc path           - Append to prefixes of files regular expressionsto include
+ -loose_perms val    - Use loose permissions on all files and directories in
+                       the coverage db so that code changing EUID can still
+                       write coverage information (default off)
+ -merge val          - Merge databases, for multiple test benches (default on)
+ -select RE          - Set regular expressions of files to select (default none)
+ +select RE          - Append to regular expressions of files to select
+ -silent val         - Don't print informational messages (default off)
+ -subs_only val      - Only cover code in subroutine bodies (default off)
+ -replace_ops val    - Use op replacing rather than runops (default on)
+ -summary val        - Print summary information if val is true (default on)
 
 =head2 More on Coverage Options
 
@@ -1407,6 +1490,15 @@ and you may specify as many as you wish.  For example, to specify that all
 subroutines containing xx are private, call Devel::Cover with the option
 -coverage,pod-also_private-xx.
 
+Or, to ignore all files in C<t/lib> as well as files ending in C<Foo.pm>:
+
+    cover -test -silent -ignore ^t/lib/,Foo.pm$
+
+Note that C<-ignore> replaces any default ignore regexes.  To preserve any
+ignore regexes which have already been set, use C<+ignore>:
+
+    cover -test -silent +ignore ^t/lib/,Foo.pm$
+
 =head1 SELECTING FILES TO COVER
 
 You may select the files for which you want to collect coverage data using the
@@ -1416,15 +1508,15 @@ decide whether a file will be included in coverage reports:
 =over
 
 =item * If the file matches a RE given as a select option, it will be
-included.
+included
 
 =item * Otherwise, if it matches a RE given as an ignore option, it won't be
-included.
+included
 
 =item * Otherwise, if it is in one of the inc directories, it won't be
-included.
+included
 
-=item * Otherwise, it will be included.
+=item * Otherwise, it will be included
 
 =back
 
@@ -1456,6 +1548,11 @@ your code and determined that you are not going to be able to exercise it, it
 may be better to record that fact in some formal fashion and stop Devel::Cover
 complaining about it, so that real problems are not lost in the noise.
 
+If you have uncoverable criteria I suggest not using the default HTML report
+(with uses html_minimal at the moment) because this sometimes shows uncoverable
+points as uncovered.  Instead, you should use the html_basic report for HTML
+output which should behave correctly in this regard.
+
 There are two ways to specify a construct as uncoverable, one invasive and one
 non-invasive.
 
@@ -1474,7 +1571,7 @@ construct.
 =head3 Statements
 
 The "uncoverable" comment should appear on either the same line as the
-statement, of on the line before it:
+statement, or on the line before it:
 
     $impossible++;  # uncoverable statement
     # uncoverable statement
@@ -1507,6 +1604,19 @@ Both branches may be uncoverable:
         handle_it_another_way();  # uncoverable statement
     }
 
+If there is an elsif in the branch then it can be addressed as the second
+branch on the line by using the "count" attribute.  Further elsifs are the
+third and fourth "count" value, and so on:
+
+# uncoverable branch false count:2
+if ($thing == 1) {
+    handle_thing_being_one();
+} elsif ($thing == 2) {
+    handle_thing_being_tow();
+} else {
+    die "thing can only be one or two, not $thing"; # uncoverable statement
+}
+
 =head3 Conditions
 
 Because of the way in which Perl short-circuits boolean operations, there are
@@ -1531,6 +1641,9 @@ conditions may be modelled thus:
 C<Or> conditionals are handled in a similar fashion (TODO - provide some
 examples) but C<xor> conditionals are not properly handled yet.
 
+As for branches, the "count" value may be used for either conditions in elsif
+conditionals, or for complex conditions.
+
 =head3 Subroutines
 
 A subroutine should be marked as uncoverable at the point where the first
@@ -1546,22 +1659,23 @@ case at the moment.
 =head2 Non-invasive specification
 
 If you can't, or don't want to add coverage comments to your code, you can
-specify the uncoverable information in a separate file.  My default this file
-is L<.uncoverable> but you can override that.
+specify the uncoverable information in a separate file.  By default the files
+PWD/.uncoverable and HOME/.uncoverable are checked.  If you use the
+-uncoverable_file parameter then the file you provide is checked as well as
+those two files.
 
 The interface to managing this file is the L<cover> program, and the options
 are:
 
  -uncoverable_file
  -add_uncoverable_point
- -delete_uncoverable_point
- -clean_uncoverable_points
+ -delete_uncoverable_point   **UNIMPLEMENTED**
+ -clean_uncoverable_points   **UNIMPLEMENTED**
 
-Of these, only the first two are implemented at the moment.  The parameter for
--add_uncoverable_point is a string composed of up to seven space separated
-elements: "$file $criterion $line $count $type $class $note".
+The parameter for -add_uncoverable_point is a string composed of up to seven
+space separated elements: "$file $criterion $line $count $type $class $note".
 
-TODO - more information and examples.
+The contents of the uncoverable file is the same, with one point per line.
 
 =head1 ENVIRONMENT
 
@@ -1572,6 +1686,13 @@ $HARNESS_PERL_SWITCHES or $PERL5OPT.  Devel::Cover tries to do the right thing
 when $MOD_PERL is set.  $DEVEL_COVER_OPTIONS is appended to any options passed
 into Devel::Cover.
 
+Note that when Devel::Cover is invoked via an environment variable, any modules
+specified on the command line, such as via the -Mmodule option, will not be
+covered.  This is because the environment variables are processed after the
+command line and any code to be covered must appear after Devel::Cover has been
+loaded.  To work around this, Devel::Cover can also be specified on the command
+line.
+
 =head2 Developer variables
 
 When running Devel::Cover's own test suite, $DEVEL_COVER_DEBUG turns on
@@ -1662,7 +1783,7 @@ Modules used by Devel::Cover while gathering coverage:
 
 =item * L<File::Spec>
 
-=item * L<Storable> or L<JSON>
+=item * L<Storable> or L<JSON::MaybeXS> (and its backend) or L<Sereal>
 
 =back
 
@@ -1687,7 +1808,7 @@ Please report new bugs on Github.
 
 =head1 LICENCE
 
-Copyright 2001-2014, Paul Johnson (paul@pjcj.net)
+Copyright 2001-2018, Paul Johnson (paul@pjcj.net)
 
 This software is free.  It is licensed under the same terms as Perl itself.
 
