@@ -1,4 +1,4 @@
-# Copyright 2014-2015, Paul Johnson (paul@pjcj.net)
+# Copyright 2014-2018, Paul Johnson (paul@pjcj.net)
 
 # This software is free.  It is licensed under the same terms as Perl itself.
 
@@ -7,15 +7,16 @@
 
 package Devel::Cover::Collection;
 
-use 5.16.0;
+use 5.26.0;
 use warnings;
 
-our $VERSION = '1.21'; # VERSION
+our $VERSION = '1.31'; # VERSION
 
 use Devel::Cover::DB;
 use Devel::Cover::DB::IO::JSON;
 use Devel::Cover::Dumper;
 
+use JSON::MaybeXS ();
 use Parallel::Iterator "iterate_as_array";
 use POSIX              "setsid";
 use Template;
@@ -27,8 +28,8 @@ use namespace::clean;
 use warnings FATAL => "all";  # be explicit since Moo sets this
 
 my %A = (
-    ro  => [ qw( bin_dir cpancover_dir cpan_dir results_dir force output_file
-                 report timeout verbose workers docker local                ) ],
+    ro  => [ qw( bin_dir cpancover_dir cpan_dir results_dir dryrun force
+                 output_file report timeout verbose workers docker local    ) ],
     rwp => [ qw( build_dirs local_timeout modules module_file               ) ],
     rw  => [ qw(                                                            ) ],
 );
@@ -38,18 +39,19 @@ sub BUILDARGS {
     my $class = shift;
     my (%args) = @_;
     {
-        build_dirs      => [],
-        cpan_dir        => [grep -d, glob("~/.cpan ~/.local/share/.cpan")],
-        docker          => "docker",
-        force           => 0,
-        local           => 0,
-        local_timeout   => 0,
-        modules         => [],
-        output_file     => "index.html",
-        report          => "html_basic",
-        timeout         => 1800,  # half an hour
-        verbose         => 0,
-        workers         => 0,
+        build_dirs    => [],
+        cpan_dir      => [grep -d, glob("~/.cpan ~/.local/share/.cpan")],
+        docker        => "docker",
+        dryrun        => 0,
+        force         => 0,
+        local         => 0,
+        local_timeout => 0,
+        modules       => [],
+        output_file   => "index.html",
+        report        => "html_basic",
+        timeout       => 1800, # half an hour
+        verbose       => 0,
+        workers       => 0,
         %args,
     }
 };
@@ -64,6 +66,7 @@ sub _sys {
     my $timeout = $self->local_timeout || $self->timeout || 30 * 60;
     my $max = 4e4;
     # say "Setting alarm for $timeout seconds";
+    my $ok = 0;
     my $pid;
     eval {
         open STDIN, "<", "/dev/null" or die "Can't read /dev/null: $!";
@@ -91,6 +94,11 @@ sub _sys {
                 }
             }
             alarm 0;
+            if (close $fh) {
+                $ok = 1;
+            } else {
+                warn "Error running @command\n";
+            }
         } else {
             setsid() != -1          or die "Can't start a new session: $!";
             open STDERR, ">&STDOUT" or die "Can't dup stdout: $!";
@@ -98,17 +106,20 @@ sub _sys {
         }
     };
     if ($@) {
-        die "propogate: $@" unless $@ eq "alarm\n";  # propagate unexpected errs
+        $ok = 0;
+        die "$@" unless $@ eq "alarm\n";  # propagate unexpected errs
         warn "Timed out after $timeout seconds!\n";
         my $pgrp = getpgrp($pid);
         my $n = kill "-KILL", $pgrp;
         warn "killed $n processes";
     }
-    length $output2 ? "$output1\n...\n$output2" : $output1
+    $ok ? length $output2 ? "$output1\n...\n$output2" : $output1 : undef
 }
 
-sub sys  { my $self = shift; $self->_sys(4e4, @_) }
-sub bsys { my $self = shift; $self->_sys(0,   @_) }
+sub sys   { my ($s, @a) = @_; $s->_sys(4e4, @a) // ""                   }
+sub bsys  { my ($s, @a) = @_; $s->_sys(0,   @a) // ""                   }
+sub fsys  { my ($s, @a) = @_; $s->_sys(4e4, @a) // die "Can't run @a\n" }
+sub fbsys { my ($s, @a) = @_; $s->_sys(0,   @a) // die "Can't run @a\n" }
 
 sub add_modules {
     my $self = shift;
@@ -148,7 +159,7 @@ sub build_modules {
     my %m;
     for my $module (sort grep !$m{$_}++, @{$self->modules}) {
         say "Building $module";
-        my $output = $self->sys(@command, $module);
+        my $output = $self->fsys(@command, $module);
         say $output;
     }
     $self->_set_local_timeout(0);
@@ -160,28 +171,29 @@ sub add_build_dirs {
     # say && system "ls -al $_" for "/remote_staging",
                                   # map "$_/build", @{$self->cpan_dir};
     my $exists = sub {
-        my $dir = "/remote_staging/" . (s|.*/||r =~ s/-\w{6}$/*/r);
-        # say "checking [$dir]";
+        # say "exists [$_]";
+        my $dir = "/remote_staging/" . (s|.*/||r =~ s/-\d+$/*/r);
         my @files = glob $dir;
+        # say "checking [$dir] -> [@files]";
         @files
     };
     push @{$self->build_dirs},
          grep { !$exists->() }
          grep -d,
          map glob("$_/build/*"), @{$self->cpan_dir};
-    # say "add_build_dirs"; say for @{$self->build_dirs};
+    # say "add_build_dirs:"; say for @{$self->build_dirs};
 }
 
 sub run {
     my $self = shift;
     my ($build_dir) = @_;
 
-    my ($module)    = $build_dir =~ m|.*/([^/]+?)(?:-\w{6})$| or return;
+    my ($module)    = $build_dir =~ m|.*/([^/]+?)(?:-\d+)$| or return;
     my $db          = "$build_dir/cover_db";
     my $line        = "=" x 80;
     my $output      = "**** Checking coverage of $module ****\n";
     my $results_dir = $self->results_dir // die "No results dir";
-    $output        .= $self->sys("mkdir", "-p", $results_dir);
+    $output        .= $self->fsys("mkdir", "-p", $results_dir);
     $results_dir   .= "/$module";
 
     chdir $build_dir or die "Can't chdir $build_dir: $!\n";
@@ -198,7 +210,8 @@ sub run {
     $output .= "Testing $module in $build_dir\n";
     # say "\n$line\n$output$line\n"; return;
 
-    # $self->sys($^X, "-V");
+    # $output .= $self->sys($^X, "-V");
+    # $output .= $self->sys("pwd");
     my @cmd;
     if ($self->local) {
         $ENV{DEVEL_COVER_OPTIONS} = "-ignore,/usr/local/lib/perl5";
@@ -207,18 +220,18 @@ sub run {
     } else {
         @cmd = ($^X, $self->bin_dir . "/cover");
     }
-    $output .= $self->bsys(
-        @cmd,          "-test",
-        "-report",     $self->report,
-        "-outputfile", $self->output_file,
+    $output .= $self->fbsys(
+        @cmd,           "--test",
+        "--report",     $self->report,
+        "--outputfile", $self->output_file,
     );
-    $output .= $self->sys(@cmd, "-report", "json", "-nosummary");
+    $output .= $self->fsys(@cmd, "-report", "json", "-nosummary");
 
     # TODO - option to merge DB with existing one
     # TODO - portability
-    $output .= $self->sys("rm", "-rf", $results_dir);
-    $output .= $self->sys("mv", $db, $results_dir);
-    $output .= $self->sys("rm", "-rf", $db);
+    $output .= $self->fsys("rm", "-rf", $results_dir);
+    $output .= $self->fsys("mv", $db, $results_dir);
+    $output .= $self->fsys("rm", "-rf", $db);
 
     say "\n$line\n$output$line\n";
 }
@@ -227,7 +240,7 @@ sub run_all {
     my $self = shift;
 
     my $results_dir = $self->results_dir // die "No results dir";
-    $self->sys("mkdir", "-p", $results_dir);
+    $self->fsys("mkdir", "-p", $results_dir);
 
     my @res = iterate_as_array(
         { workers => $self->workers },
@@ -377,12 +390,80 @@ sub generate_html {
             or die $template->error;
     }
 
+    my $about_f = "$d/about.html";
+    say "\nWriting about page to $about_f ...";
+
+    $template->process("about", { subdir => "latest/" }, $about_f)
+        or die $template->error;
+
     # print Dumper $vars;
     $self->write_json($vars);
 
     say "Wrote collection output to $f";
 }
 
+sub compress_old_versions {
+    my $self = shift;
+    my ($versions) = @_;
+
+    my $dir = $self->results_dir;
+    opendir my $fh, $dir or die "Can't opendir $dir: $!";
+    my @dirs = sort grep -d, map "$dir/$_", readdir $fh;
+    closedir $fh or die "Can't closedir $dir: $!";
+
+    my %modules;
+    for my $dir (@dirs) {
+        my $file = "$dir/cover.json";
+        my $json = JSON::MaybeXS->new(utf8 => 1, allow_blessed => 1);
+        open my $fh, "<", $file or next;
+        # say "file: $file";
+        my $data = do { local $/; eval { $json->decode(<$fh>) } } or next;
+        next if $@;
+        close $fh or next;
+        my ($name)  = $dir =~ /.+\/(.+)/;
+        $name =~ s/-[^-]+$//;
+        my @runs    = grep { ($_->{name} // "") eq $name } $data->{runs}->@*;
+        # say "$name " . @runs;
+        my $run     = $runs[0]                   // next;
+        my $version = $run->{version} =~ s/_//gr // next;
+        my $v       = eval { version->parse($version)->numify };
+        if ($@ || !$v) {
+            $v = $version;
+            $v =~ s/[^0-9.]//g;
+            my @parts = split /\./, $v;
+            if (@parts > 2) {
+                $v = shift(@parts) . "." . join "", @parts;
+            }
+        }
+        $v ||= 0;
+        push $modules{$name}->@*, { dir => $dir, version => $v };
+    }
+
+    for my $name (sort keys %modules) {
+        # print Dumper $modules{$name};
+        my @o = sort { $b->{version} <=> $a->{version} } $modules{$name}->@*;
+        shift @o for 1 .. $versions;
+        for my $v (@o) {
+            my ($d, $s) = $v->{dir} =~ /(.+)\/(.+)/;
+            my $archive = "$v->{dir}.tar.xz";
+            my @cmd1    = ($self->dc_file, "-r", $d,
+                           "cpancover-uncompress-dir", $s);
+            my @cmd2    = ("bash", "-c",
+                           "tar cf - -C $d $s | xz -z > $archive");
+            my @cmd3    = ("rm", "-rf", $v->{dir});
+            if ($self->dryrun) {
+                say for "compressing $s", "@cmd1", "@cmd2", "@cmd3";
+            } else {
+                say "compressing $s";
+                eval {
+                    $self->fsys(@$_) for \@cmd1, \@cmd2, \@cmd3;
+                };
+                say $@ if $@;
+            }
+        }
+    }
+}
+
 sub local_build {
     my $self = shift;
 
@@ -390,7 +471,6 @@ sub local_build {
     $self->build_modules;
     $self->add_build_dirs;
     $self->run_all;
-    $self->generate_html;
 }
 
 sub failed_dir {
@@ -439,14 +519,22 @@ sub set_failed {
     close $fh or warn "Can't close $ff: $!";
 }
 
+sub dc_file {
+    my $self = shift;
+    my $dir = "";
+    $dir = "/dc/" if $self->local && -d "/dc";
+    "${dir}utils/dc"
+}
+
 sub cover_modules {
     my $self = shift;
 
     $self->process_module_file;
-
     # say "modules: ", Dumper $self->modules;
 
-    my @command = qw( utils/dc cpancover-docker-module );
+    my @cmd = $self->dc_file;
+    push @cmd, "--local" if $self->local;
+    my @command = (@cmd, "cpancover-docker-module");
     $self->_set_local_timeout(0);
     my @res = iterate_as_array(
         { workers => $self->workers },
@@ -458,10 +546,10 @@ sub cover_modules {
             if ($self->is_covered($dir)) {
                 $self->set_covered($dir);
                 say "$module already covered" if $self->verbose;
-                return;
+                return unless $self->force;
             } elsif ($self->is_failed($dir)) {
                 say "$module already failed" if $self->verbose;
-                return;
+                return unless $self->force;
             }
 
             my $timeout = $self->local_timeout || $self->timeout || 30 * 60;
@@ -472,11 +560,12 @@ sub cover_modules {
             eval {
                 local $SIG{ALRM} = sub { die "alarm\n" };
                 alarm $timeout;
+                say "running: @command $module $name" if $self->verbose;
                 system @command, $module, $name;
                 alarm 0;
             };
             if ($@) {
-                die "propogate: $@" unless $@ eq "alarm\n";  # unexpected errors
+                die "$@" unless $@ eq "alarm\n";  # unexpected errors
                 say "Timed out after $timeout seconds!";
                 $self->sys($self->docker, "kill", $name);
                 say "Killed docker container $name";
@@ -614,7 +703,7 @@ package Devel::Cover::Collection::Template::Provider;
 use strict;
 use warnings;
 
-our $VERSION = '1.21'; # VERSION
+our $VERSION = '1.31'; # VERSION
 
 use base "Template::Provider";
 
@@ -651,7 +740,7 @@ $Templates{html} = <<'EOT';
 <html xmlns="http://www.w3.org/1999/xhtml">
 <!--
 This file was generated by Devel::Cover Version $VERSION
-Devel::Cover is copyright 2001-2015, Paul Johnson (paul\@pjcj.net)
+Devel::Cover is copyright 2001-2018, Paul Johnson (paul\@pjcj.net)
 Devel::Cover is free. It is licensed under the same terms as Perl itself.
 The latest version of Devel::Cover should be available from my homepage:
 http://www.pjcj.net
@@ -665,8 +754,8 @@ http://www.pjcj.net
 </head>
 <body>
     [% content %]
-    <p/>
     <hr/>
+    <p>
     Coverage information from <a href="https://metacpan.org/module/Devel::Cover">
       Devel::Cover
     </a> by <a href="http://pjcj.net">Paul Johnson</a>.
@@ -674,15 +763,15 @@ http://www.pjcj.net
     <br/>
 
     Please report problems with this site to the
-    <a href="https://github.com/pjcj/Devel--Cover/issues">issue tracker</a>.
+    <a href="https://github.com/pjcj/Devel--Cover/issues">issue tracker</a>.</p>
 
-    <br/>
-    <br/>
+    <p><a href="http://cpancover.com/latest/about.html">About</a> the project.</p>
 
-    This server generously donated by
+    <p>This server generously donated by
     <a href="http://www.bytemark.co.uk/r/cpancover">
       <img src="http://www.bytemark.co.uk/images/subpages/spreadtheword/bytemark_logo_179_x_14.png" alt="bytemark"/>
     </a>
+    </p>
 </body>
 </html>
 EOT
@@ -694,7 +783,7 @@ $Templates{summary} = <<'EOT';
 
 <h2> Distributions </h2>
 
-Search for distributions by first character:<p/>
+<p>Search for distributions by first character:</p>
 
 [% FOREACH start = modules.keys.sort %]
     <a href="dist/[%- start -%].html">[% start %]</a>
@@ -708,6 +797,32 @@ Search for distributions by first character:<p/>
 [% END %]
 EOT
 
+$Templates{about} = <<'EOT';
+[% WRAPPER html %]
+
+<h1> CPANCover </h1>
+
+<h2> About </h2>
+
+<p>CPANCover is a project to provide code coverage information for <a
+href="https://metacpan.org"> CPAN</a> modules.  When a new module, or an update to an
+existing module, is uploaded to CPAN it will automatically be downloaded by
+CPANCover.  CPANCover will run the module's tests and measure the code coverage
+provided by the tests.  This information is then made available as HTML pages
+and JSON data.</p>
+
+<p>The coverage data is generated by <a
+href="https://metacpan.org/release/Devel-Cover">Devel::Cover</a>.
+
+<p>The source code is available at the
+  <a href="https://github.com/pjcj/Devel--Cover">GitHub repository</a>.
+  Contributions are also accepted for several
+  <a href="http://pjcj.net/devel-cover/projects.html">open projects</a>.
+</p>
+
+[% END %]
+EOT
+
 $Templates{module_by_start} = <<'EOT';
 [% WRAPPER html %]
 
@@ -769,7 +884,7 @@ Devel::Cover::Collection - Code coverage for a collection of modules
 
 =head1 VERSION
 
-version 1.21
+version 1.31
 
 =head1 SYNOPSIS
 
@@ -785,7 +900,7 @@ Almost certainly.
 
 =head1 LICENCE
 
-Copyright 2014-2015, Paul Johnson (paul@pjcj.net)
+Copyright 2014-2018, Paul Johnson (paul@pjcj.net)
 
 This software is free.  It is licensed under the same terms as Perl itself.
 
