@@ -1,4 +1,4 @@
-# Copyright 2004-2017, Paul Johnson (paul@pjcj.net)
+# Copyright 2004-2018, Paul Johnson (paul@pjcj.net)
 
 # This software is free.  It is licensed under the same terms as Perl itself.
 
@@ -20,7 +20,7 @@ use Devel::Cover::Dumper;
 # For comprehensive debug logging.
 use constant DEBUG => 0;
 
-our $VERSION = '1.29'; # VERSION
+our $VERSION = '1.31'; # VERSION
 our $AUTOLOAD;
 
 sub new {
@@ -337,7 +337,7 @@ sub read_all {
 sub merge {
     my $self   = shift;
     my ($from) = @_;
-    Devel::Cover::DB::_merge_hash($self->{f}, $from->{f});
+    Devel::Cover::DB::_merge_hash($self->{f}, $from->{f}, "noadd");
 }
 
 1
@@ -350,7 +350,7 @@ Devel::Cover::DB::Structure - Internal: abstract structure of a source file
 
 =head1 VERSION
 
-version 1.29
+version 1.31
 
 =head1 SYNOPSIS
 
@@ -371,7 +371,7 @@ Huh?
 
 =head1 LICENCE
 
-Copyright 2004-2017, Paul Johnson (paul@pjcj.net)
+Copyright 2004-2018, Paul Johnson (paul@pjcj.net)
 
 This software is free.  It is licensed under the same terms as Perl itself.
 
