@@ -1,4 +1,4 @@
-# Copyright 2001-2015, Paul Johnson (paul@pjcj.net)
+# Copyright 2001-2018, Paul Johnson (paul@pjcj.net)
 
 # This software is free.  It is licensed under the same terms as Perl itself.
 
@@ -10,13 +10,18 @@ package Devel::Cover::Report::Html_basic;
 use strict;
 use warnings;
 
-our $VERSION = '1.21'; # VERSION
-our $LVERSION = do { eval '$VERSION' || "0.001" };  # for development purposes
+our $VERSION;
+BEGIN {
+our $VERSION = '1.31'; # VERSION
+}
 
 use Devel::Cover::DB;
 use Devel::Cover::Html_Common "launch";
+use Devel::Cover::Inc;
 use Devel::Cover::Web "write_file";
 
+BEGIN { $VERSION //= $Devel::Cover::Inc::VERSION }
+
 use HTML::Entities;
 use Getopt::Long;
 use Template 2.00;
@@ -398,7 +403,7 @@ sub report {
         perl_v  => $] < 5.010 ? $] : $^V,
         os      => $^O,
         options => $options,
-        version => $LVERSION,
+        version => $VERSION,
         showing => [ grep $options->{show}{$_}, $db->criteria ],
         headers => [
             map { ($db->criteria_short)[$_] }
@@ -422,7 +427,6 @@ sub report {
     );
 
     write_file $R{options}{outputdir}, "all";
-    my $html = print_summary;
 
     for (@{$options->{file}}) {
         $R{file} = $_;
@@ -435,6 +439,7 @@ sub report {
         print_subroutines if $show->{subroutine} || $show->{pod};
     }
 
+    my $html = print_summary;
     print "HTML output written to $html\n" unless $options->{silent};
 }
 
@@ -445,7 +450,7 @@ package Devel::Cover::Report::Html_basic::Template::Provider;
 use strict;
 use warnings;
 
-our $VERSION = '1.21'; # VERSION
+our $VERSION = '1.31'; # VERSION
 
 use base "Template::Provider";
 
@@ -465,7 +470,7 @@ $Templates{html} = <<'EOT';
 <html xmlns="http://www.w3.org/1999/xhtml">
 <!--
 This file was generated by Devel::Cover Version [% R.version %]
-Devel::Cover is copyright 2001-2012, Paul Johnson (paul@pjcj.net)
+Devel::Cover is copyright 2001-2018, Paul Johnson (paul@pjcj.net)
 Devel::Cover is free. It is licensed under the same terms as Perl itself.
 The latest version of Devel::Cover should be available from my homepage:
 http://www.pjcj.net
@@ -804,7 +809,7 @@ Devel::Cover::Report::Html_basic - HTML backend for Devel::Cover
 
 =head1 VERSION
 
-version 1.21
+version 1.31
 
 =head1 SYNOPSIS
 
@@ -833,7 +838,7 @@ Huh?
 
 =head1 LICENCE
 
-Copyright 2001-2015, Paul Johnson (paul@pjcj.net)
+Copyright 2001-2018, Paul Johnson (paul@pjcj.net)
 
 This software is free.  It is licensed under the same terms as Perl itself.
 
