@@ -1,4 +1,4 @@
-# Copyright 2002-2017, Paul Johnson (paul@pjcj.net)
+# Copyright 2002-2018, Paul Johnson (paul@pjcj.net)
 
 # This software is free.  It is licensed under the same terms as Perl itself.
 
@@ -10,7 +10,7 @@ package Devel::Cover::Test;
 use strict;
 use warnings;
 
-our $VERSION = '1.30'; # VERSION
+our $VERSION = '1.31'; # VERSION
 
 use Carp;
 
@@ -303,7 +303,7 @@ sub run_cover {
     if ($self->{differences}) {
         no warnings "redefine";
         local *Test::_quote = sub { "@_" };
-        $self->{no_coverage} ? pass : eq_or_diff(\@at, \@ac, "output");
+        $self->{no_coverage} ? pass : eq_or_diff(\@at, \@ac, "output", { context => 0 });
     } elsif ($self->{no_coverage}) {
         pass for @{$self->{cover}};
     }
@@ -390,7 +390,7 @@ Devel::Cover::Test - Internal module for testing
 
 =head1 VERSION
 
-version 1.30
+version 1.31
 
 =head1 METHODS
 
@@ -489,7 +489,7 @@ Huh?
 
 =head1 LICENCE
 
-Copyright 2001-2017, Paul Johnson (paul@pjcj.net)
+Copyright 2001-2018, Paul Johnson (paul@pjcj.net)
 
 This software is free.  It is licensed under the same terms as Perl itself.
 
